@@ -11,6 +11,7 @@
 #include "postgres.h"
 
 #include "access/htup_details.h"
+#include "access/parallel.h"
 #include "access/sysattr.h"
 #include "access/xact.h"
 #include "access/relation.h"
@@ -41,7 +42,7 @@
 #include "utils/timestamp.h"
 #include "utils/varlena.h"
 
-PG_MODULE_MAGIC;
+PG_MODULE_MAGIC_EXT(.name = "pgaudit", .version = "18.0");
 
 void _PG_init(void);
 
@@ -289,6 +290,18 @@ static int64 stackTotal = 0;
 
 static bool statementLogged = false;
 
+/*
+ * Check that the stack is not empty for hooks that add data to an audit event
+ * that was started by the ProcessUtility or ExecutorCheckPerms hooks.
+ *
+ * We are unable to continue in this case without losing an audit record. If the
+ * caller is purposefully breaking the hook sequence then they will need to
+ * disable auditing for the duration of the operation.
+ */
+ #define STACK_NOT_EMPTY() \
+    if (auditEventStack == NULL) \
+        elog(ERROR, "pgaudit stack is empty");
+
 /*
  * Stack functions
  *
@@ -316,7 +329,7 @@ stack_free(void *stackFree)
             /* Move top of stack to the item after the freed item */
             auditEventStack = nextItem->next;
 
-            /* If the stack is not empty */
+            /* If the stack is now empty */
             if (auditEventStack == NULL)
             {
                 /*
@@ -383,11 +396,7 @@ stack_push()
                                        &stackItem->contextCallback);
 
     /* Push new item onto the stack */
-    if (auditEventStack != NULL)
-        stackItem->next = auditEventStack;
-    else
-        stackItem->next = NULL;
-
+    stackItem->next = auditEventStack;
     auditEventStack = stackItem;
 
     MemoryContextSwitchTo(contextOld);
@@ -440,13 +449,8 @@ stack_find_context(MemoryContext findContext)
     AuditEventStackItem *nextItem = auditEventStack;
 
     /* Look through the stack for the stack entry by query memory context */
-    while (nextItem != NULL)
-    {
-        if (nextItem->auditEvent.queryContext == findContext)
-            break;
-
+    while (nextItem != NULL && nextItem->auditEvent.queryContext != findContext)
         nextItem = nextItem->next;
-    }
 
     return nextItem;
 }
@@ -521,9 +525,8 @@ append_valid_csv(StringInfoData *buffer, const char *appendStr)
     if (appendStr == NULL)
         return;
 
-    /* Only format for CSV if appendStr contains: ", comma, \n, \r */
-    if (strstr(appendStr, ",") || strstr(appendStr, "\"") ||
-        strstr(appendStr, "\n") || strstr(appendStr, "\r"))
+    /* Only format for CSV if appendStr contains: comma, ", \n, \r */
+    if (strpbrk(appendStr, ",\"\n\r"))
     {
         appendStringInfoCharMacro(buffer, '"');
 
@@ -616,8 +619,8 @@ log_audit_event(AuditEventStackItem *stackItem)
                  */
                 case T_CreateRoleStmt:
                 case T_AlterRoleStmt:
-				case T_CreateUserMappingStmt:
-				case T_AlterUserMappingStmt:
+                case T_CreateUserMappingStmt:
+                case T_AlterUserMappingStmt:
 
                     if (stackItem->auditEvent.commandText != NULL)
                     {
@@ -643,6 +646,7 @@ log_audit_event(AuditEventStackItem *stackItem)
                             passwordPos = (passwordToken - commandStr) +
                                           strlen(TOKEN_PASSWORD);
 
+                            pfree(commandStr);
                             commandStr = palloc(passwordPos + 1 +
                                                 strlen(TOKEN_REDACTED) + 1);
 
@@ -659,6 +663,8 @@ log_audit_event(AuditEventStackItem *stackItem)
                             stackItem->auditEvent.commandText = commandStr;
                             stackItem->auditEvent.commandLen = strlen(commandStr);
                         }
+                        else
+                            pfree(commandStr);
                     }
 
                 /* Fall through */
@@ -1295,6 +1301,16 @@ log_select_dml(Oid auditOid, List *rangeTabls, List *permInfos)
                                                perminfo->updatedCols,
                                                auditPerms);
             }
+
+            /*
+             * SELECT FOR UPDATE is not logged when SELECT is not granted
+             */
+            if (rte->rellockmode == RowShareLock &&
+                !audit_on_relation(relOid, auditOid, ACL_SELECT))
+                auditEventStack->auditEvent.granted =
+                    audit_on_any_attribute(relOid, auditOid,
+                                           perminfo->selectedCols,
+                                           ACL_SELECT);
         }
 
         /* Do relation level logging if a grant was found */
@@ -1402,9 +1418,9 @@ pgaudit_ExecutorStart_hook(QueryDesc *queryDesc, int eflags)
 {
     AuditEventStackItem *stackItem = NULL;
 
-    if (!internalStatement)
+    if (!internalStatement && !IsParallelWorker())
     {
-        /* Push the audit even onto the stack */
+        /* Push the audit event onto the stack */
         stackItem = stack_push();
 
         /* Initialize command using queryDesc->operation */
@@ -1476,7 +1492,9 @@ pgaudit_ExecutorStart_hook(QueryDesc *queryDesc, int eflags)
  * Hook ExecutorCheckPerms to do session and object auditing for DML.
  */
 static bool
-pgaudit_ExecutorCheckPerms_hook(List *rangeTabls, List *permInfos, bool abort)
+pgaudit_ExecutorCheckPerms_hook(List *rangeTabls,
+                                List *permInfos,
+                                bool ereport_on_violation)
 {
     Oid auditOid;
 
@@ -1485,7 +1503,7 @@ pgaudit_ExecutorCheckPerms_hook(List *rangeTabls, List *permInfos, bool abort)
 
     /* Log DML if the audit role is valid or session logging is enabled */
     if ((auditOid != InvalidOid || auditLogBitmap != 0) &&
-        !IsAbortedTransactionBlockState())
+        !IsAbortedTransactionBlockState() && !IsParallelWorker())
     {
         /* If auditLogRows is on, wait for rows processed to be set */
         if (auditLogRows && auditEventStack != NULL)
@@ -1506,8 +1524,8 @@ pgaudit_ExecutorCheckPerms_hook(List *rangeTabls, List *permInfos, bool abort)
             else
             {
                 /*
-                 * Save auditOid and rangeTabls to call log_select_dml()
-                 * in pgaudit_ExecutorEnd_hook() later.
+                 * Save auditOid, rangeTabls, and permInfos to call
+                 * log_select_dml() in pgaudit_ExecutorEnd_hook() later.
                  */
                 auditEventStack->auditEvent.auditOid = auditOid;
                 auditEventStack->auditEvent.rangeTabls = rangeTabls;
@@ -1515,12 +1533,16 @@ pgaudit_ExecutorCheckPerms_hook(List *rangeTabls, List *permInfos, bool abort)
             }
         }
         else
+        {
+            STACK_NOT_EMPTY();
             log_select_dml(auditOid, rangeTabls, permInfos);
+        }
     }
 
     /* Call the next hook function */
     if (next_ExecutorCheckPerms_hook &&
-        !(*next_ExecutorCheckPerms_hook) (rangeTabls, permInfos, abort))
+        !(*next_ExecutorCheckPerms_hook)(rangeTabls, permInfos,
+                                         ereport_on_violation))
         return false;
 
     return true;
@@ -1530,24 +1552,27 @@ pgaudit_ExecutorCheckPerms_hook(List *rangeTabls, List *permInfos, bool abort)
  * Hook ExecutorRun to get rows processed by the current statement.
  */
 static void
-pgaudit_ExecutorRun_hook(QueryDesc *queryDesc, ScanDirection direction, uint64 count, bool execute_once)
+pgaudit_ExecutorRun_hook(QueryDesc *queryDesc, ScanDirection direction, uint64 count)
 {
     AuditEventStackItem *stackItem = NULL;
 
     /* Call the previous hook or standard function */
     if (next_ExecutorRun_hook)
-        next_ExecutorRun_hook(queryDesc, direction, count, execute_once);
+        next_ExecutorRun_hook(queryDesc, direction, count);
     else
-        standard_ExecutorRun(queryDesc, direction, count, execute_once);
+        standard_ExecutorRun(queryDesc, direction, count);
 
-    if (auditLogRows && !internalStatement)
+    if (auditLogRows && !internalStatement && !IsParallelWorker())
     {
         /* Find an item from the stack by the query memory context */
         stackItem = stack_find_context(queryDesc->estate->es_query_cxt);
 
         /* Accumulate the number of rows processed */
         if (stackItem != NULL)
+        {
+            STACK_NOT_EMPTY();
             stackItem->auditEvent.rows += queryDesc->estate->es_processed;
+        }
     }
 }
 
@@ -1560,13 +1585,15 @@ pgaudit_ExecutorEnd_hook(QueryDesc *queryDesc)
     AuditEventStackItem *stackItem = NULL;
     AuditEventStackItem *auditEventStackFull = NULL;
 
-    if (auditLogRows && !internalStatement)
+    if (auditLogRows && !internalStatement && !IsParallelWorker())
     {
         /* Find an item from the stack by the query memory context */
         stackItem = stack_find_context(queryDesc->estate->es_query_cxt);
 
         if (stackItem != NULL && stackItem->auditEvent.rangeTabls != NULL)
         {
+            STACK_NOT_EMPTY();
+
             /* Reset auditEventStack to use in log_select_dml() */
             auditEventStackFull = auditEventStack;
             auditEventStack = stackItem;
